@@ -20,7 +20,7 @@ void zandroid_get_location(double *h, double *w, int *state);
 void zandroid_play_wav(const char *filename);
 void zandroid_ssbd_abort(void);
 void zandroid_messagebox(const char *caption, const char *str);
-void zandroid_update_location();
+void zandroid_update_location(void);
 
 
 #endif
