@@ -53,7 +53,8 @@ struct zzsdl {
 	iconv_t to8b_cd; 
 	GString *to8b_gs;
 #ifdef Z_HAVE_SDL_TTF
-	TTF_Font* ttf;
+	GHashTable* ttfs;// key=height, of TTF_Font *
+	SDL_RWops *ttf_fontrw; // foreign key from app, do not free in libzia!
 	iconv_t ttf_iconv;
 #endif
 };
