@@ -69,6 +69,25 @@ static struct zzsdl *zsdl = NULL;
 
 static short *outline9x16;
 
+#ifdef Z_HAVE_SDL_TTF
+static void z_free_ttf_font(gpointer xxx) {
+    TTF_Font* ttf = (TTF_Font*)xxx;
+    TTF_CloseFont(ttf);
+}
+
+static TTF_Font* z_sdl_get_ttf(int font_h) {
+    int pt = (int)(font_h * 0.75) + 1;
+
+    TTF_Font* ret = g_hash_table_lookup(zsdl->ttfs, GINT_TO_POINTER(font_h));
+    if (ret == NULL) {
+        SDL_RWseek(zsdl->ttf_fontrw, 0, RW_SEEK_SET);
+        ret = TTF_OpenFontRW(zsdl->ttf_fontrw, 0, pt);
+        if (ret != NULL) g_hash_table_insert(zsdl->ttfs, GINT_TO_POINTER(font_h), ret);
+    }
+    return ret;
+}
+
+#endif
 
 struct zzsdl *zsdl_init(SDL_Surface *screen, int font_h, int inverse, SDL_RWops* fontRW){
 
@@ -106,17 +125,28 @@ struct zzsdl *zsdl_init(SDL_Surface *screen, int font_h, int inverse, SDL_RWops*
 	zsdl->antialiasing = 0;
     //zsdl->antialiasing_supported = 0;
 
+#ifndef Z_HAVE_SDL_TTF
     zsdl->font_h = font_h;
     zsdl->font_w = zsdl_h2w(font_h);
+#endif
+
 #ifdef Z_HAVE_SDL_TTF
+    zsdl->font_h = 16;
+    zsdl->font_w = 9; // initial ratio for zsdl_h2w
+    zsdl->font_w = zsdl_h2w(font_h);
+    zsdl->font_h = font_h;
+
     TTF_Init();
     if (fontRW != NULL) {
-        int w, h, pt = (int)(font_h * 0.75) + 1;
-        zsdl->ttf = TTF_OpenFontRW(fontRW, 0, pt);
-        if (zsdl->ttf != NULL) {
-            TTF_SizeText(zsdl->ttf, "W", &w, &h);
+        int w, h; 
+        zsdl->ttf_fontrw = fontRW;
+        if (!zsdl->ttfs) zsdl->ttfs = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, z_free_ttf_font);
+        TTF_Font *ttf = z_sdl_get_ttf(font_h);
+        if (ttf != NULL) {
+            TTF_SizeText(ttf, "W", &w, &h);
             zsdl->font_w = w;
             zsdl->font_h = h;
+            dbg("zsdl_init: pt=%d font_w=%d  font_h=%d\n", (int)(font_h * 0.75) + 1, zsdl->font_w, zsdl->font_h);
         }
     }
     zsdl->ttf_iconv = iconv_open("UTF-8", "ISO8859-2");
@@ -146,12 +176,12 @@ void zsdl_free(void){
 	if (zsdl->pxformat != NULL) SDL_FreeFormat(zsdl->pxformat);
 #endif
 #ifdef Z_HAVE_SDL_TTF
-    if (zsdl->ttf != NULL) TTF_CloseFont(zsdl->ttf);
+    g_hash_table_destroy(zsdl->ttfs);
     iconv_close(zsdl->ttf_iconv);
 #endif
 	if (zsdl->to8b_cd != NULL) iconv_close(zsdl->to8b_cd);
 	zsdl->to8b_cd = NULL;
-	g_string_free(zsdl->to8b_gs, TRUE);
+	if (zsdl->to8b_gs != NULL) g_string_free(zsdl->to8b_gs, TRUE);
 
     g_free(zsdl);
     zsdl = NULL;
@@ -162,6 +192,10 @@ struct zzsdl *zsdl_instance(){
 }
 
 int zsdl_h2w(int font_h){
+#ifdef Z_HAVE_SDL_TTF
+    int ret = (zsdl->font_w * font_h) / zsdl->font_h;
+    return ret;
+#else
 	switch (font_h){
 		case 13:
 			return 8;
@@ -180,6 +214,7 @@ int zsdl_h2w(int font_h){
 		default:
 			return (font_h * 9) / 16;
 	}
+#endif
 }
 
 SDL_Surface *z_create_surface(struct zzsdl *zsdl, int w, int h){
@@ -211,10 +246,19 @@ void SDL_UpdateRect(SDL_Surface *surface, int x, int y, int w, int h){
 #endif
 
 int zsdl_max_font_h(int w, int h, char *text){
-	int ww;
 	int l = strlen(text);
 
-	ww = zsdl_h2w(64) * l;
+#ifdef Z_HAVE_SDL_TTF
+    float rx = (float)w / (l * zsdl->font_w);
+    float ry = (float)h / zsdl->font_h;
+    float r = rx < ry ? rx : ry;
+    int fh = (int)floor(r * zsdl->font_h);
+    return fh;
+
+#else
+    int ww;
+
+    ww = zsdl_h2w(64) * l;
 	if (h >= 64 && w >= ww) return 64;
 
 	ww = zsdl_h2w(48) * l;
@@ -233,6 +277,7 @@ int zsdl_max_font_h(int w, int h, char *text){
 	if (h >= 16 && w >= ww) return 16;
 
 	return 13; 
+#endif
 }
 
 short *zfont_create_outline(const short *font, int size_b, int font_h){
@@ -505,20 +550,60 @@ static int sbits[]={1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 819
 
 
 #if defined(Z_HAVE_SDL_TTF) && !defined(Z_HAVE_SDL2)
-// very limited, only vertical stretch, only srcrect=NULL (taken from src)
-int SDL_BlitScaled(SDL_Surface* src, const SDL_Rect* srcrect, SDL_Surface* dst, SDL_Rect* dstrect) {
-    for (int y = 0; y < dstrect->h; y++) {
-        int srcy = (y * src->h) / dstrect->h;
-        SDL_Rect s = { .x = 0, .y = srcy, .w = src->w, .h = 1 };
-        SDL_Rect d = { .x = dstrect->x, .y = dstrect->y + y, .w = dstrect->w, .h = 1 };
-        SDL_BlitSurface(src, &s, dst, &d);
+int SDL_BlitScaled(SDL_Surface* srcA, SDL_Rect* srcrect, SDL_Surface* dst, SDL_Rect* dstrect) {
+    if (!srcA || !dst) {
+        return -1;
     }
+
+    // TTF outputs pixes that differs in alpha
+    SDL_Surface *src = SDL_CreateRGBSurface(SDL_SWSURFACE, srcA->w, srcA->h,
+        dst->format->BitsPerPixel,
+        dst->format->Rmask,
+        dst->format->Gmask,
+        dst->format->Bmask,
+        0); // without alpha
+    SDL_BlitSurface(srcA, NULL, src, NULL);
+
+    SDL_Rect full_src = { 0, 0, src->w, src->h };
+    SDL_Rect full_dst = { 0, 0, dst->w, dst->h };
+
+    if (!srcrect) {
+        srcrect = &full_src;
+    }
+    if (!dstrect) {
+        dstrect = &full_dst;
+    }
+
+    int dstW = dstrect->w;
+    int dstH = dstrect->h;
+
+    double scaleX = (double)dstW / srcrect->w;
+    double scaleY = (double)dstH / srcrect->h;
+
+    SDL_LockSurface(src);
+    SDL_LockSurface(dst);
+
+    for (int y = 0; y < dstH; ++y) {
+        for (int x = 0; x < dstW; ++x) {
+            int srcX = srcrect->x + (int)(x / scaleX);
+            int srcY = srcrect->y + (int)(y / scaleY);
+
+            int c = z_getpixel(src, srcX, srcY);
+            z_putpixel(dst, dstrect->x + x, dstrect->y + y, c);
+        }
+    }
+
+    SDL_UnlockSurface(src);
+    SDL_UnlockSurface(dst);
+    SDL_FreeSurface(src);
+
+    return 0;
 }
+
 #endif
 
 int zsdl_print_ttf_char(SDL_Surface* surface, int *x, int y, int color, int bgcolor, int flags, unsigned char c) {
 #ifdef Z_HAVE_SDL_TTF
-    if (zsdl->ttf == NULL) return 0;
     if (c < 0x20) return 0;
     if (c >= 0x7f && c <= 0x9f) return 0;
     //if (c == 0xff) return 0;
@@ -559,16 +644,22 @@ int zsdl_print_ttf_char(SDL_Surface* surface, int *x, int y, int color, int bgco
 
     if ((flags & ZFONT_TRANSP) == 0) SDL_FillRect(surface, &r, bgcolor);
 
-    SDL_Surface *src = TTF_RenderUTF8_Blended(zsdl->ttf, utf, fg);
+    int h = zsdl->font_h;
+    if (flags & (ZFONT_DOUBLEHB | ZFONT_DOUBLEHT)) h *= 2;
+
+    TTF_Font* ttf = z_sdl_get_ttf(h);
+    if (ttf == NULL) return 0;
+    SDL_Surface *src = TTF_RenderUTF8_Blended(ttf, utf, fg);
    
     if (flags & ZFONT_OUTLINE) {
         flags &= ~(ZFONT_DOUBLEHB | ZFONT_DOUBLEHT);
-        TTF_SetFontOutline(zsdl->ttf, 1);
-        SDL_Surface *outline = TTF_RenderUTF8_Blended(zsdl->ttf, utf, bg);
+        ttf = z_sdl_get_ttf(zsdl->font_h);
+        TTF_SetFontOutline(ttf, 1);
+        SDL_Surface *outline = TTF_RenderUTF8_Blended(ttf, utf, bg);
         SDL_Rect ro = { .x = *x - 1, .y = y - 1 , .w = zsdl->font_w, .h = zsdl->font_h };
         SDL_BlitSurface(outline, NULL, surface, &ro);
         SDL_FreeSurface(outline);
-        TTF_SetFontOutline(zsdl->ttf, 0);
+        TTF_SetFontOutline(ttf, 0);
     }
 
     if (flags & ZFONT_DOUBLEHT) {
@@ -579,7 +670,7 @@ int zsdl_print_ttf_char(SDL_Surface* surface, int *x, int y, int color, int bgco
         r.y -= zsdl->font_h;
         SDL_BlitScaled(src, NULL, surface, &r);
     }else if (flags & ZFONT_USERHFLG) {
-        SDL_BlitScaled(src, NULL, surface, &r);
+        SDL_BlitSurface(src, NULL, surface, &r);
     }else{
         SDL_BlitSurface(src, NULL, surface, &r);
     }
@@ -615,8 +706,10 @@ void zsdl_printf(SDL_Surface *surface, int x, int y, int color, int bgcolor, int
 	if (flags & ZFONT_USERHFLG)	{
 		old_font_w = zsdl->font_w;
 		old_font_h = zsdl->font_h;
-		zsdl->font_h = (flags >> 16) & 0xff;
-		zsdl->font_w = zsdl_h2w(zsdl->font_h);
+        int fh = (flags >> 16) & 0xff;
+		zsdl->font_w = zsdl_h2w(fh);
+        zsdl->font_h = fh;
+
 	}	
 	font_h = zsdl->font_h;
 	    
